from .condition import Condition
from .tools import peer, random_id
from .exception import VkErr
from .payload import payload
from .dot_dict import DotDict
from .upload import (
    Document
)
from .api import (
    Api, LongPoll
)
from .vkui import (
    Keyboard, Button,
    Template, Element,
)

from vk_dev import cond
